// CalibrateView.cpp : implementation file
//

#include "stdafx.h"
#include "winwapwad.h"
#include "winwadoc.h"
#include "CalibrateView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCalibrateView

IMPLEMENT_DYNCREATE(CCalibrateView, CView)

const static int CALIB_DIST = 80;	// Length of a side of our calibration square

CCalibrateView::CCalibrateView()
{
	left = MmToXPixels (10);
	top = MmToYPixels (10);
	width = MmToXPixels ((float) CALIB_DIST);
	height = MmToYPixels ((float) CALIB_DIST);
}

CCalibrateView::~CCalibrateView()
{
}


BEGIN_MESSAGE_MAP(CCalibrateView, CView)
	//{{AFX_MSG_MAP(CCalibrateView)
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCalibrateView drawing

void CCalibrateView::OnDraw(CDC* pDC)
{
	CWinwapwadDoc* pDoc = (CWinwapwadDoc*) GetDocument();
	pDC->SelectObject (pDoc->pGreyBrush);
	pDC->Rectangle (left, top, left + width, top + height);

	// Now calculate the area we need for the text and draw it
	CRect clientArea;
	GetClientRect (clientArea);
	CRect textBounds (CPoint (0, 0), clientArea.Size ());
	char* text = "Use the arrow keys to adjust the size of the square until it is exactly 80 mm.\nPress S to save the value, ESC to cancel";
	pDC->DrawText (text, -1, & textBounds, DT_CALCRECT | DT_WORDBREAK | DT_LEFT);
	CRect textArea (CPoint (10,  min (clientArea.Height () - textBounds.Height (), top + height + 20)),
					textBounds.Size ());
	pDC->DrawText (text, -1, & textArea, DT_WORDBREAK | DT_LEFT);
}

/////////////////////////////////////////////////////////////////////////////
// CCalibrateView diagnostics

#ifdef _DEBUG
void CCalibrateView::AssertValid() const
{
	CView::AssertValid();
}

void CCalibrateView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCalibrateView message handlers

void CCalibrateView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch (nChar)
	{
	case VK_LEFT:
		if (width > (int) nRepCnt + 1)
			width -= nRepCnt;
		break;
		
	case VK_RIGHT:
		width += nRepCnt;
		break;
		
	case VK_UP:
		if (height > (int) nRepCnt + 1)
			height -= nRepCnt;
		break;
		
	case VK_DOWN:
		height += nRepCnt;
		break;

	case 'S':
		((CWinwapwadDoc*) GetDocument ())->CalibrationDone ((width + 1.0) / (float) CALIB_DIST,
															(height + 1.0) / (float) CALIB_DIST);
		return;

	case VK_ESCAPE:
	case 'Q':
		((CWinwapwadDoc*) GetDocument ())->CalibrationCancel ();
		return;
		
	default:
		break;
	}
	Invalidate ();
}

