/*************************************************************************
 FILE:				DT_DEF.H     
 CREATION DATE:		14 May 1996
 LAST MODIFIED:		15 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file defines the Data_Task_Def class, which encapsulates the classes
 that those interested in extending WAPWAD need to redefine.  In particular,
 Data_Task_Def subclasses consist of four essential methods that must be
 overridden:  get_description, get_td_generator(), get_ti_generator(), and 
 get_tp_generator.  get_description returns a brief textual description
 of the particular task supported by this data_task-def class (e.g., 
 discrete pointing, discrete dragging).
 get_td_generator returns an instance of a class capable of generating
 trial_instances; the boolean parameter determines whether or not 
 trial_data instances capable of storing movement microstructure are
 generated. Similarly, get_ti_generator and 
 get_tp_generator return instances capable of generating instances of the 
 appropriate trial_interpreter and trial_presenter classes.   
*************************************************************************/  

#ifndef __DT_DEF_H
#define __DT_DEF_H

#include "boolean.h"
#include "list.h"
#include "td_gen.h"
#include "ti_gen.h"
#include "tp_gen.h"

class data_task_def {
  public:
  	virtual char* get_description();
	virtual td_generator* get_td_generator(boolean micro);
  	virtual ti_generator* get_ti_generator();
  	virtual tp_generator* get_tp_generator();
  };

#endif


