/*************************************************************************
 FILE:				DTD_DB.H     
 CREATION DATE:		14 May 1996
 LAST MODIFIED:		26 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file defines the DTD_Database class, which is responsible for 
 maintaining a table that maps names of tasks, as stored in a session_
 generator file, to instances of class data_task_def, which can generate  
 appropriate trial_data, ti_generator, and tp_generator instances for 
 that task.
*************************************************************************/  

#ifndef __DTD_DB_H
#define __DTD_DB_H

//#include "boolean.h"
#include "t_data.h"
#include "t_presen.h"
#include "list.h"

class dtd_database {
  private:
    list lookup_list; // We'll implement the lookup table as a list, since 
    				  // it is unlikely that there will ever be more than
    				  // five mappings to look up.
  public:   
  	// The data_task_def struct stores four instance generators, which
  	// the session_config class uses to generate sessions and to provide
  	// the main program with the appropriate trial_presenter class for
  	// presenting session trials.
  	struct data_task_def { 
  		char label[40];
  		trial_data* (*perfTrialGen) (int diam, int dist, int ang, bool collectTrail); 
  		trial_data* micro_trial_data;
  		trial_presenter* perf_tp;
  		trial_presenter* micro_tp;
  	};
  	
    dtd_database();
	//********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Initialize an empty database instance.
    // RETURN : An empty database instance.
    //********************************************************************
	
	BOOL add_mapping(dtd_database::data_task_def *d_t_d);
	//********************************************************************
    // GIVEN  : A data_task_def to add to the database.
    // TASK   : Add a new label->data_task_def mapping.	Warning: 
	//			Add duplicates to the database at your own risk!  
	//			We promise no checking for duplicates
    // RETURN : TRUE if label/data_task_def pair was added, FALSE otherwise.
    //********************************************************************
	
	dtd_database::data_task_def* retrieve_mapping(const char *lbl);
	//********************************************************************
    // GIVEN  : A char* lbl whose corresponding data_task_def is sought.
    // TASK   : Look up lbl in the database.
    // RETURN : data_task_def instance corresponding to lbl, or NULL if 
	//			lbl couldn't be found in the database.
    //********************************************************************
    
    list* get_all_labels();
    //********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Look up all labels in the database.
    // RETURN : A pointer to a list of all labels in the database.
    //********************************************************************
    
	~dtd_database();
	//********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Return heap space associated with this.
    // RETURN : Nothing.
    //********************************************************************
	};
 		   			
#endif 				   
