/*************************************************************************
 FILE:				EX_SES.H     
 CREATION DATE:		18 May 1996
 LAST MODIFIED:		18 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 Class experimentor_session specializes the session class by accommodating 
 experimentor sessions, which proceed until the experimentor explictly
 aborts them.  In other words, each trial that is dequeued is immediately
 requeued, to facilitate infinite trial lengths. 
*************************************************************************/  

#ifndef __EX_SES_H
#define __EX_SES_H

#include "session.h"

class experimentor_session : public session {
	public:
		friend session_config;
		// METHODS
		trial_data* get_next_trial();
		//********************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Fetch the next trial in the experimentor session. 
    	//			Immediately requeue the trial in the pending_trials,
    	//			thus ensuring that the experimentor will never run out
    	//			of trials.
    	// RETURN : A pointer to the next trial in the experimentor session.
    	//********************************************************************

		void log_trial(trial_data *td);
		//********************************************************************
    	// GIVEN  : A completed trial.
    	// TASK   : Log completed trial.
    	// RETURN : Nothing.
    	//********************************************************************


	 protected:
		// METHODS
		experimentor_session(int num_trials); 
		//********************************************************************
    	// GIVEN  : The number of trials with which to initialize the session.
    	// TASK   : Initialize a new instance of experimentor_session.
    	// RETURN : A pointer to a fresh instance of practice_session.
    	// NOTE   : Always initialize instances with
    	//			a full block's worth of trials; by requeuing
    	//			completed trials, the session will make sure that 
    	//			experimentors don't run out of trials 
    	//********************************************************************

};
		
#endif
