// MODULE  : list.cpp
// PURPOSE : This module implements the list ADT as defined in list.h
// AUTHOR  : Christopher D. Hundhausen
// DATE    : 20 June 1991

#include "stdafx.h"
#include "list.h"
#include <iostream.h>
  
ostream& operator<< (ostream &os, list &l) {
  	if (l.first()) { 
  		os << (*((int*)(l.current())));
  		while (l.next())
  			os << ", " << (*((int*)(l.current())));
  	}
  	return os; 
  }			
  	
  list::list() // constructor
  {
  head = new listnode;
  head->dataptr = NULL; // initialize dummy header
  head->prev = head;
  head->next = head;
  curr = NULL; // i.e. undefined     
  len = 0;
  }

  list::list(const list& l) // copy constructor
   {
   head = new listnode;
   head->dataptr = NULL; // initialize dummy header
   head->prev = head;
   head->next = head;
   curr = NULL;
   len = l.len;
   listnode* temp = l.head->next;
   while (temp->dataptr != NULL)
     {
     listnode* n = new listnode;
     n->dataptr = temp->dataptr;
     n->prev = head->prev;
     n->prev->next = n;
     n->next = head;
     head->prev = n;
     if (l.curr == temp)
       curr = n; // set current pointer
     temp = temp->next;
     }
  }

  list:: ~list() // destructor
   {
     listnode *temp = head->next;
     while (temp->dataptr != NULL)
       {
       delete temp->dataptr;
       temp = temp->next;
       delete temp->prev; // this ought to work
       }
     delete head; // i.e. temp at this point
   }

  void list::operator = (const list &l) // deep copy
    {
    delete this;  // destroy this
    list temp(l);
    (*this) = temp;
    }

  int list::length() //return number of items in list
  	{
  	return len;     
  	}
  	
  void* list::current() // retrieve contents of current node
    {
    if (curr == NULL)
      return NULL;
    else
      return (curr->dataptr);
    }

  int list::empty() // determine whether list is empty
    {
    return (head->next == head);
    }
 
  int list::append(void *item) // append item to tail of list
    {
    if (item == NULL) // || curr == NULL)  cannot add a NULL pointer to list,
      return 0;  // nor can we add an item to a list
    //undefined current node
    else
      {
      listnode *temp = new listnode;
      temp->dataptr = item; 
      temp->prev = head->prev;
      temp->next = head;
      temp->prev->next = temp;
      head->prev = temp;  
      ++len;
      return 1;
      }
   }

  int list::attach(void *item) // attach item to head of list
   {
   if (item == NULL)
	return 0;
    else
	  {
	  listnode *temp = new listnode;
	  temp->dataptr = item;
	  temp->prev = head;
	  temp->next = head->next;
	  temp->next->prev = temp;
	  head->next = temp;  
	  ++len;
	  return 1;
          }
  }
 
  int list::insert(void *item) // insert item before current node
   {
   if (item == NULL || curr == NULL) // cannot insert a NULL pointer into
	 return 0;  // list, or into a list w/undefined
                   // current node.
   else
	{
	listnode *temp = new listnode;
	temp->dataptr = item;
	temp->prev = curr->prev;
	temp->next = curr;
	temp->prev->next = temp;
	curr->prev = temp;    
	++len;
	return 1;
	}
  }

  int list::replace(void *item) // replace current node with item
   {
   if (item == NULL || curr == NULL)
     return 0;
   else
     {
     delete curr->dataptr; // destruct current data
     curr->dataptr = item;
     return 1;
     }
   }

  int list::remove() // remove current node from list
  // current node becomes curr->prev
   {
   if (curr == NULL)
     return 0; // cannot remove item from list w/undefined current node!
   else
     {
     curr->prev->next = curr->next;
     curr->next->prev = curr->prev;
     if (curr->next->dataptr != NULL) //make next the current node...
       {
       listnode* temp = curr;
       curr = curr->next;
       delete temp;
       }
     else
       if (curr->prev->dataptr != NULL) //make prev the current node...
	 {
	   listnode* temp = curr;
           curr = curr->prev;
	   delete temp;
	   }
        else // empty list -- curr now undefined
          {
          delete curr;
          curr = NULL;
	} 
	--len;
    return 1;
   }
  }

  int list::first()  // advance current to head of list
  {
  if (head->next == head)  // empty list
    return 0;
  else
    {
    curr = head->next;
    return 1;
    }
  }

  int list::last() // advance current to tail of list
  {
  if (head->next == head) // empty list
    return 0;
  else
    {
      curr = head->prev;
      return 1;
    }
   }

  int list::prev() // move current to previous node
  {
  if (curr == NULL || curr->prev->dataptr == NULL) // curr undefined or already at start of list
    return 0;
  else
    {
    curr = curr->prev;
    return 1;
    }
  }

  int list::next()  // advance current to next node
   {
   if ((curr == NULL) || (curr->next->dataptr == NULL))  // curr undefined or already at end of list
     return 0;
   else
     {
     curr = curr->next;
     return 1;
     }
   }

