// HEADER  : list.h
// PURPOSE : This module provides the definition for the list abstract data
//           type.

#ifndef __LIST_H
#define __LIST_H

#include <fstream.h>



class list
  {
 
  protected:
    struct listnode
      {
      listnode * prev;
      listnode * next;
      void * dataptr;
      };
    listnode *head, *curr;
    int len;
  
  public:  
  	friend ostream& operator<<(ostream& os, list &l);
  	// Allows us to do cout << list.  Note that we assume
  	// that lists contain integers here -- a quick and
  	// dirty faus paux.
  	
    list();
    //*************************************************************************
    // GIVEN  : Nothing.
    // TASK   : Initialize an empty list.
    //***********************************************************************

    list(const list &l); // copy constructor
    //***********************************************************************
    // GIVEN  : A list object 'l.'
    // TASK   : Initialize "this" to a copy of 'l.'
    //***********************************************************************

    ~list(); 
    //***********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Return all heap space associated with "this."
    //***********************************************************************

    void operator = (const list &l);
    //***********************************************************************
    // GIVEN  : A list object 'l.'
    // TASK   : Set "this" to point to a copy of 'l.'
    //***********************************************************************
     
    int length();  
    //***********************************************************************
    // GIVEN  : Nothing
    // RETURN : The number of elements on the list.
    //***********************************************************************
   
    
    void* current();
    //***********************************************************************
    // GIVEN  : Nothing.
    // RETURN : Contents of "current" node, if it is defined; NULL otherwise.
    //***********************************************************************

    int empty();
    //***********************************************************************
    // GIVEN  : Nothing
    // RETURN : 1 if list is empty, 0 otherwise.
    //***********************************************************************

    //***********************************************************************
    // NOTE:  All of the following functions return the result of the
    //        operation as their respective function result (0 = failure,
    //        1 = uccess)
    //***********************************************************************

    int append(void *item);
    //***********************************************************************
    // GIVEN  : An 'item' to append to the list.
    // TASK   : Append 'item' to tail of the list.
    //***********************************************************************

    int attach(void *item);
    //***********************************************************************
    // GIVEN  : An 'item' to attach to the list.
    // TASK   : Attach 'item to head of the list.
    //***********************************************************************

    int insert(void *item);
    //***********************************************************************
    // GIVEN  : An 'item' to insert in the list.
    // TASK   : If "current node is defined, insert 'item' before
    //          "current" node.
    //***********************************************************************

    int replace(void *item);
    //***********************************************************************
    // GIVEN  : An 'item' with which to replace data currently in the list.
    // TASK   : If "current" node is defined, replace data in "current"
    //          node with 'item.'
    //***********************************************************************

    int remove();
    //***********************************************************************
    // GIVEN  : Nothing.
    // TASK   : If "current" node is defined, remove it from list.
    //***********************************************************************

    int first();
    //***********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Advance "current" to head of the list.
    // NOTE 1 : 'first' only works on non-empty lists.
    // NOTE 2 : You may use first to DEFINE the current node.
    //***********************************************************************

    int last();   // advance current to tail of list
    //***********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Advance "current" to tail of the list.
    // NOTE 1 : 'last' only works on non-empty lists.
    // NOTE 2 : You may use last to DEFINE the current node.
    //***********************************************************************

    int prev();
    //***********************************************************************
    // GIVEN  : Nothing.
    // TASK   : If possible (i.e. if "current" is defined and isn't already
    //          at the head of the list), advance "current" to previous node.
    //***********************************************************************

    int next();
    //***********************************************************************
    // GIVEN  : Nothing.
    // TASK   : If possible (i.e. if "current" is defined and isn't already
    //          at the tail of the list), advance "current" to next node.
    //***********************************************************************
  };

#endif

