/*************************************************************************
 FILE:				PA_SES.H     
 CREATION DATE:		18 May 1996
 LAST MODIFIED:		18 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 Class participant_session specializes the session class by accommodating 
 participant block sessions, which have the following distinguishing
 characteristics:  (1) a fixed length, in terms of 
 a number of trials; and (2) an error trial handling policy, which is 
 passed to the class constructor. 
*************************************************************************/  

#ifndef __PA_SES_H
#define __PA_SES_H

#include "session.h"

class participant_session : public session {
	public:
		friend session_config;

		enum ehandling_techniques {redo_at_end, dont_redo};

		// METHODS
		trial_data* get_next_trial();
		//********************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Fetch next trial in the block, possibly looking at the
    	//			redo_trials queue if the pending_trials queue is empty
    	// RETURN : Pointer to next trial to present.
    	//********************************************************************
	
		void log_trial(trial_data *td); // log trial    
		//********************************************************************
    	// GIVEN  : A pointer to a trial that has just been completed.
    	// TASK   : Register trial as having been completed, requeuing it
    	//			if the error policy dictates.
    	//********************************************************************
	
		~participant_session(); //destructor
        //********************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Return all heap space associated with this.
    	//********************************************************************
	
	 protected:
	 	// DATA  
	 	ehandling_techniques err_tech;
		pqueue redo_trials; //this queue facilitates the redo_at_end error handling technique.
		
		// METHODS
		participant_session(int num_trials, int blocks, int seed, ehandling_techniques eht,
			ofstream* out, ofstream* trails, const CString& participantID, const CString& sessNum); 
		//********************************************************************
    	// GIVEN  : A random number seed and the error handling policy with which
		//			to initialize a participant session.
    	// TASK   : Initialize a new instance of participant_session with the
    	//			desired error handling policy.
    	// RETURN : A pointer to a fresh instance of participant_session.
    	//********************************************************************

};
		
#endif
