/*************************************************************************
 FILE:				PQUEUE.C     
 CREATION DATE:		5 May 1996
 LAST MODIFIED:		7 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 In this file, we implement the priority queue ADT defined in pqueue.h.
*************************************************************************/
#include "stdafx.h"
#include "pqueue.h"

//! Just for debugging
#include <iostream.h>        
#include <fstream.h>

pqueue::pqueue(int max_elements, int seed) {
		
		// GIVEN:  An integer indicating the maximum number of elements that
		//         will be stored in the queue, and a seed with which to 
		//		   initialize the queue's pseudorandom number generator.
		// TASK:   Class constructor -- inititialize an instance of pqueue
		//		   appropriately.
		// RETURN: Fresh instance of queue.   
	srand(seed); // Initialize random number generator with seed
	max_elts = max_elements;   
	size = 0; // No elements in queue initially
	queue_array = new vptr[max_elts];
}
         
int pqueue::enqueue(void* elt) {  

		// GIVEN:  Pointer to element to enqueue.
		// TASK:   Place element in queue.
		// RETURN: 1 if element was successfully enqueued, 0 otherwise.

	if (size <  max_elts) {  // queue has space; enqueue element at queue_array[size]
		queue_array[size] = elt;     
		++size;   // We've added an element; increment size of queue  
		return 1;
		}
	else // queue is full
		return 0;
}
	

void* pqueue::dequeue() {
		// GIVEN:  Nothing
		// TASK:   Use random priority function to select element to dequeue
		// RETURN: Pointer to element that was dequeued, or NULL if the 
		//		   queue is empty.       
		int hit;
		void* temp;
		if (size == 0)  // Queue is empty
			return NULL;
		else  { // We can dequeue an elt. . .     
			// First, select the element to deuqueue using the rand() function. . .
			//!  For debugging, just take hindmost
			//hit = size - 1;
			hit = rand() % size;  
			// Second, grab the element. . .
			temp = queue_array[hit];
			// Third, fill in the space that the element occupied with the last
			// element in the array, thereby shinking the size of the array by 1. . .
			queue_array[hit] = queue_array[size-1];
			// Finally, decrement the number of the elts in the queue
			--size; 
			return temp;		
		} //else
}	
	
int pqueue::empty() {
		// GIVEN:  Nothing
		// TASK:   Determine whether the queue is empty.
		// RETURN: TRUE if queue is empty, FALSE otherwise.
		return (size == 0);
}

int pqueue::full() {
		// GIVEN:  Nothing
		// TASK:   Determine whether the queue has reached its maximum capacity.
		// RETURN: TRUE if queue is empty, FALSE otherwise.
		return (size == max_elts);
}

pqueue::~pqueue() {
		// GIVEN:  Nothing.
		// TASK:   Class destructor -- Reclaim heap space allocated to this 
		//		   instance
		// RETURN: Nothing.                     
	delete queue_array;
}
