/*************************************************************************
 FILE:				PQUEUE.H     
 CREATION DATE:		5 May 1996
 LAST MODIFIED:		7 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 In this file, we define the priority queue ADT that will be used to generate
 sequences of trials.  Trials are placed into the queue with the standard
 enqueue function, and removed from the queue with the standard dequeue
 function.  The dequeue function makes use of a priority function to select
 the next element to be dequeued; we will define that function such that
 it randomly selects the element to be dequeued.
*************************************************************************/
#ifndef __PQUEUE_H

#define __PQUEUE_H
#include <stdlib.h>       

class pqueue {
	private: 
		typedef void* vptr;                         
		int max_elts, size;
		vptr *queue_array;  
			  
	public:
		pqueue(int max_elements, int seed);
		// GIVEN:  An integer indicating the maximum number of elements that
		//         will be stored in the queue, and a seed with which to 
		//		   initialize the queue's pseudorandom number generator.
		// TASK:   Class constructor -- inititialize an instance of pqueue
		//		   appropriately.
		// RETURN: Fresh instance of queue.
		
		int enqueue(void* elt);  
		// GIVEN:  Pointer to element to enqueue.
		// TASK:   Place element in queue.
		// RETURN: 1 if element was successfully enqueued, 0 otherwise.
		
		void* dequeue();  
		// GIVEN:  Nothing
		// TASK:   Use random priority function to select element to dequeue
		// RETURN: Pointer to element that was dequeued, or NULL if the 
		//		   queue is empty.

		int empty();
		// GIVEN:  Nothing
		// TASK:   Determine whether the queue is empty.
		// RETURN: 1 if queue is empty, 0 otherwise.
	
		int full();
		// GIVEN:  Nothing
		// TASK:   Determine whether the queue has reached its maximum capacity.
		// RETURN: 1 if queue is full, 0 otherwise.
	
		~pqueue(); 
		// GIVEN:  Nothing.
		// TASK:   Class destructor -- Reclaim heap space allocated to this 
		//		   instance
		// RETURN: Nothing.                     
		
};

#endif