/*************************************************************************
 FILE:				PR_SES.H     
 CREATION DATE:		18 May 1996
 LAST MODIFIED:		18 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 Class practice_session specializes the session class by accommodating 
 practice sessions, which are bounded either by a set number of trials, or
 by a set time.  The class constructor takes the length and units of the
 session as parameters.
*************************************************************************/  

#ifndef __PR_SES_H
#define __PR_SES_H

#include <time.h>
#include "session.h"

class practice_session : public session {
	public:
		friend session_config;
		enum psession_units {trials, minutes};
		// Is practice session length specified in terms of a number of trials, or in
		// terms of a number of minutes?
		// METHODS
		trial_data* get_next_trial();
		//********************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Fetch the next trial in the practice session. If the 
    	//			session is bounded by time, then immediately requeue 
    	//			the trial in the pending_trials queue; this
    	//			ensures that we'll never run out of practice trials.
    	// RETURN : A pointer to the next trial in the practice session, if
    	//			time or trials have not run out, and NULL if time or
    	//			trials have run out.
    	//********************************************************************

		void log_trial(trial_data *td);
		//********************************************************************
    	// GIVEN  : A completed trial.
    	// TASK   : Log completed trial.
    	// RETURN : Nothing.
    	//********************************************************************

	 protected:							  
	 	// DATA
		int session_len;
		time_t session_start_time;  // What was the clock time at which the session started?
		BOOL first_trial_requested; // Has the first trial of the session been requested?
		psession_units psu;

		// METHODS
		practice_session(int num_trials, int ses_len, psession_units u); 
		//********************************************************************
    	// GIVEN  : The number of trials with which to initialize the session,
    	//			the length of the practice session, and the units in which
    	//			that length is expressed.
    	// TASK   : Initialize a new instance of practice_session with the
    	//			desired length.
    	// RETURN : A pointer to a fresh instance of practice_session.
    	// NOTE   : Be sure always to initialize instances of practice_
    	//			session with a full block's worth of trials, regardless
    	//			of the length of the practice session. If the units of 
    	//			the session length are trials, then that number of 
    	//			trials will be randomly selected from all block 
    	//			trials.  If, on the other hand, the units of the 
    	//			session length are minutes,the session will make sure that 
    	//			participants don't run out of practice trials by requeuing
    	//			completed trials, 
    	//********************************************************************

};

#endif		
