// HEADER  : queue.h
// PURPOSE : Defines the operations on the queue ADT.

#ifndef __QUEUE_H
#define __QUEUE_H

#include <fstream.h>

class queue
  {
 
  protected:
    struct queuenode
      {
      queuenode *next;
      void  *dataptr;
      };
      queuenode *front, *rear;
  
  public:
    queue();
    //**********************************************************************
    // GIVEN  : Nothing
    // TASK   : Initialize an empty queue.
    //**********************************************************************

    queue(const queue &q);
    //**********************************************************************
    // GIVEN  : A queue object 'q.'
    // TASK   : Initialize "this" to be a copy of 'q.'
    //**********************************************************************

    ~queue();
    //**********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Return all heap space associated with "this."
    //**********************************************************************

    void operator = (const queue &q);
    //**********************************************************************
    // GIVEN  : a queue object 'q.'
    // TASK   : assign "this" to a copy of 'q.'
    //**********************************************************************

    int empty();
    //**********************************************************************
    // GIVEN  : Nothing.
    // RETURN : 0 if the queue is empty, 1 otherwise.
    //**********************************************************************

    void enqueue(void *item);
    //**********************************************************************
    // GIVEN  : An 'item' to be added to the queue.
    // TASK   : Insert 'item' at the rear of the queue.
    //**********************************************************************

    void* dequeue();
    //**********************************************************************
    // GIVEN  : Nothing.
    // TASK   : Remove an item from the head of the queue
    // RETURN : The item at the head of the queue, or NULL if the queue is
    //          empty.
    //**********************************************************************
  };

#endif