/*************************************************************************
 FILE:				S_CONFIG.CPP  
 CREATION DATE:		10 May 1996
 LAST MODIFIED:		21 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file implements the session_config class, as defined in s_config.h.
 ************************************************************************/ 

#include "stdafx.h"
#include <string.h>  
#include <ctype.h>
#include <math.h>

#include "t_data.h"

#include "s_config.h" 

int session_config::my_atoi(const char *str) {
	int tmp = atoi(str);
	if (tmp != 0) // positive integer
		return tmp;
	else if (str[0] == '0')	 // 0 is OK
		return 0;
	else  // The number really is garbage
		return -1;
}

double session_config::my_atof(const char *str) {
	double tmp = atof(str);
	if (tmp != 0.0) // positive float
		return tmp;
	else if (str[0] == '0')	 // 0 is OK
		return 0.0;
	else  // The number really is garbage
		return -1.0;
}

void session_config::absorb_comment(ifstream &inf) {
	char c; 
	do {
		inf.get(c);
	} while ((c != '\n') && (!inf.eof()));
}

BOOL session_config::eatwhite(ifstream &inf) {
	char c;
	while (inf.get(c)) {
		if (isspace(c)==0) {
			inf.putback(c);
			return FALSE;
		} 
		else if (c == '\n')
			return TRUE;	
	}
	return TRUE;
}

BOOL session_config::read_val(ifstream &inf, char *val) {
	char c;   
	int pos = 0;
	while ((pos < 79) && (inf.get(c))) {
		if (isspace(c)) {
			inf.putback(c);
			break;
		}
		else 
			val[pos++] = c;
	}   
	if (pos == 80)
		return FALSE;
	else  {
		val[pos] = '\0'; // Properly terminate the val.
		return TRUE;
	}
}
	
		
session_config::keyword session_config::read_kw_and_vals(ifstream &inf, int c_line, ostrstream& s_mes) {
	char c;                              
	int *i;
	int int_val;
	double double_val;
	char kw[26]; // No present keyword is longer than 25 chars  
	char val[80]; // The values associated with a keyword       
	BOOL newline_encountered, value_good;
	int num_kw_chars = 0;
	int vals_read = 0;
	//First, try to read the keyword. . .    	
	while ((num_kw_chars < 25) && (inf.get(c)))
		if (c == ':')
			break;
		else
			kw[num_kw_chars++] = c;
	if (num_kw_chars == 25) {
		s_mes << "ERROR:  Keyword on line " << c_line 
			  << " is not properly terminated with a ':'.\n";
		return error;
	}
	//If we're here, the keyword is at least of the right length;
	//let's explore further. . .
	kw[num_kw_chars] = '\0';  // Terminate string we just read in
	if (newline_encountered = eatwhite(inf)) { ; // Get to first val. . .
		s_mes << "ERROR:  Unexpected end-of-line encountered on line " 
			  << c_line << ".\n";
		return error;
	}
	//If we're here, we can proceed with trying to read in values. . .
	if (_stricmp(kw,"task_type")==0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			dtd = dtd_db->retrieve_mapping(val); 
			if (dtd) { //mapping was found!   
				while (eatwhite(inf) == FALSE); //read to end of line    
				return task_type;
			}
			else {
				s_mes << "ERROR: Value for TASK_TYPE keyword on line " <<
				c_line << " not recognized.\n";
				return error;
			}
		}
		else {
			s_mes << "ERROR:  Value associated with keyword TASK_TYPE on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}	
	} // "task"

	if (_stricmp(kw,"target_diameters")==0) { 
		//Need to read in a series of whitespace-separated integers. . . 
		newline_encountered = eatwhite(inf);
		while (newline_encountered == FALSE) { // There's a val to get
			value_good = read_val(inf,val); 
			if (value_good == TRUE)  {
				// Try to convert val to a double. . . 
				if ((double_val = my_atof(val)) >= 0.0F) {    
					i = new int (double_val * DIST_UNIT + 0.5);
					target_diams.append(i);
					newline_encountered = eatwhite(inf);
				}                                        
				else {
					s_mes << "ERROR: Non-numeric value associated " <<
					"with keyword TARGET_DIAMETERS on line " << c_line << ".\n";
					return error;
				}	
			}	
			else {
				s_mes << "ERROR:  Value associated with keyword TARGET_DIAMETERS on" <<
				" line " << c_line << "not properly terminated.\n";
				return error;
			}
		} // while
		return target_diameters;
	} // "target_diameters"
	if (_stricmp(kw,"target_distances") == 0) {
		//Need to read in a series of whitespace-separated integers. . . 
		newline_encountered = eatwhite(inf);
		while (newline_encountered == FALSE) { // There's a val to get
			value_good = read_val(inf,val); 
			if (value_good == TRUE)  {
				// Try to convert val to a double. . . 
				if ((double_val = my_atof(val)) >= 0.0F) {    
					i = new int (double_val * DIST_UNIT + 0.5);
					target_dists.append(i);
					newline_encountered = eatwhite(inf);
				}                                        
				else {
					s_mes << "ERROR: Non-numeric value associated " <<
					"with keyword TARGET_DISTANCES on line " << c_line << ".\n";
					return error;
				}
			}		
			else {
				s_mes << "ERROR:  Value associated with keyword TARGET_DISTANCES on" <<
				" line " << c_line << "not properly terminated.\n";
				return error;
			}
		} // while
		return target_distances;
	}
	if (_stricmp(kw,"target_angles") == 0) { 
		//Need to read in a series of whitespace-separated integers. . . 
		newline_encountered = eatwhite(inf);
		while (newline_encountered == FALSE) { // There's a val to get
			value_good = read_val(inf,val); 
			if (value_good == TRUE)  {
				// Try to convert val to an integer. . . 
				if ((int_val = my_atoi(val)) >= 0) {    
					i = new int;
					(*i) = int_val;   
					target_angs.append(i);
					newline_encountered = eatwhite(inf);
				}                                        
				else {
					s_mes << "ERROR: Non-numeric value associated " <<
					"with keyword TARGET_ANGLES on line " << c_line << ".\n";
					return error;
				}
			}		
			else {
				s_mes << "ERROR:  Value associated with keyword TARGET_ANGLES on" <<
				" line " << c_line << "not properly terminated.\n";
				return error;
			}
		} // while
		return target_angles;
		} 
	if (_stricmp(kw,"trials_per_combination") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (int_val = atoi(val)) { //valid value  
				trials_per_comb = int_val;
				while (eatwhite(inf) == FALSE);
				return trials_per_combination;
			}
			else {
				s_mes << "ERROR: Non-integer value for TRIALS_PER_COMBINATION keyword on line " <<
				c_line << ".\n";
				return error;
			}
		}
		else {
			s_mes << "ERROR:  Value associated with keyword TRIALS_PER_COMBINATION on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}	
	} // "trials_per_combination"
	if (_stricmp(kw,"number_of_blocks") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (int_val = atoi(val)) { //valid value  
				num_blocks = int_val;
				while (eatwhite(inf) == FALSE);
				return number_of_blocks;
			}
			else {
				s_mes << "ERROR: Non-integer value for NUMBER_OF_BLOCKS keyword on line " <<
				c_line << ".\n";
				return error;
			}
		}
		else {
			s_mes << "ERROR:  Value associated with keyword NUMBER_OF_BLOCKS on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}	
	} // "number_of_blocks"
    if (_stricmp(kw,"practice_session_length") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (int_val = atoi(val)) { //valid value  
				ps_len = int_val;
				while (eatwhite(inf) == FALSE);
				return practice_session_length;
			}
			else {
				s_mes << "ERROR: Non-integer value for PRACTICE_SESSION_LENGTH keyword on line " <<
				c_line << ".\n";
				return error;
			}
		}
		else {
			s_mes << "ERROR:  Value associated with keyword PRACTICE_SESSION_LENGTH on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}	
	} // "practice_session_length"
	if (_stricmp(kw,"error_handling_technique") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (_stricmp(val,"redo_at_end") == 0) {
				err_tech = participant_session::redo_at_end;
				while (eatwhite(inf) == FALSE);
				return error_handling_technique;
			}  
			if (_stricmp(val,"dont_redo") == 0) {
				err_tech = participant_session::dont_redo;
				while (eatwhite(inf) == FALSE);
				return error_handling_technique;
			}  
			s_mes << "ERROR:  Invalid value for ERROR_HANDLING_TECHNIQUE keyword on line " <<
				  c_line << ".\n"; 
			return error;
		}
		else {
			s_mes << "ERROR:  Value associated with keyword TRIALS_PER_COMBINATION on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}	
	}
	if (_stricmp(kw,"practice_session_units") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (_stricmp(val,"trials") == 0) {
				ps_units = practice_session::trials;
				while (eatwhite(inf) == FALSE);
				return practice_session_units;
			}  
			if (_stricmp(val,"minutes") == 0) {
				ps_units = practice_session::minutes;
				while (eatwhite(inf) == FALSE);
				return practice_session_units;
			}  
			s_mes << "ERROR:  Invalid value for PRACTICE_SESSION_UNITS keyword on line " <<
				  c_line << ".\n"; 
			return error;
		}
		else {
			s_mes << "ERROR:  Value associated with keyword PRACTICE_SESSION_UNITS on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}
	}  
	if (_stricmp(kw,"home_square_loc") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (_stricmp(val,"center") == 0) {
				home = center;
				while (eatwhite(inf) == FALSE);
				return home_square_loc;
			}  
			if (_stricmp(val,"random") == 0) {
				home = random;
				while (eatwhite(inf) == FALSE);
				return home_square_loc;
			}  
			s_mes << "ERROR:  Invalid value for HOME_SQUARE_LOC keyword on line " <<
				  c_line << ".\n"; 
			return error;
		}
		else {
			s_mes << "ERROR:  Value associated with keyword HOME_SQUARE_LOC on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}
	} 	
	if (_stricmp(kw,"data_to_collect") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (_stricmp(val,"performance") == 0) {
				collect_data = performance;
				while (eatwhite(inf) == FALSE);
				return data_to_collect;
			}  
			if (_stricmp(val,"micro") == 0) {
				collect_data = micro;
				while (eatwhite(inf) == FALSE);
				return data_to_collect;
			}  
			s_mes << "ERROR:  Invalid value for DATA_TO_COLLECT keyword on line " <<
				  c_line << ".\n"; 
			return error;
		}
		else {
			s_mes << "ERROR:  Value associated with keyword DATA_TO_COLLECT on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}
	}
	if (_stricmp(kw,"save_practice_data") == 0) {
		value_good = read_val(inf,val);
		if (value_good) { // Need to make sure value is valid
			if (_stricmp(val,"yes") == 0) {
				save_pdata = TRUE;
				while (eatwhite(inf) == FALSE);
				return save_practice_data;
			}  
			if (_stricmp(val,"no") == 0) {
				save_pdata = FALSE;
				while (eatwhite(inf) == FALSE);
				return save_practice_data;
			}  
			s_mes << "ERROR:  Invalid value for SAVE_PRACTICE_DATA keyword on line " <<
				  c_line << ".\n"; 
			return error;
		}
		else {
			s_mes << "ERROR:  Value associated with keyword SAVE_PRACTICE_DATA on" <<
			" line " << c_line << "not properly terminated.\n";
			return error;
		}
	}
	// If we get here, the keyword was unrecognized. . .
	s_mes << "ERROR:  Unrecognized keyword " << kw << " on line " << c_line
		  << ".\n";
	return error; 
 }
      
void session_config::fill_in_missing_kw(keyword key) {
	int *i;
	
	switch (key) {
		case task_type:        
		   	// Just grab the first label on the list. . .
			labels_list->first();
			dtd = dtd_db->retrieve_mapping((char *)labels_list->current());
			break;	
		case error_handling_technique:
			err_tech = participant_session::redo_at_end;
			break;
		case practice_session_units:
			ps_units = practice_session::trials;
			break;
		case practice_session_length:
			ps_len = 10;
			break;
		case target_diameters:  
			i = new int; (*i) = 2 * DIST_UNIT; target_diams.append(i);
			i = new int; (*i) = 4 * DIST_UNIT; target_diams.append(i);
			i = new int; (*i) = 8 * DIST_UNIT; target_diams.append(i); 
			break;
		case target_distances:
			i = new int; (*i) = 16 * DIST_UNIT; target_dists.append(i);
			i = new int; (*i) = 32 * DIST_UNIT; target_dists.append(i);
			i = new int; (*i) = 64 * DIST_UNIT; target_dists.append(i);
			break;
		case target_angles:
			i = new int; (*i) = 0; target_angs.append(i);
			i = new int; (*i) = 90; target_angs.append(i);
			i = new int; (*i) = 180; target_angs.append(i);	
			i = new int; (*i) = 270; target_angs.append(i); 
			break;
		case number_of_blocks:
			num_blocks = 1;
			break;
		case data_to_collect:
			collect_data = performance;
			break;
		case home_square_loc:
			home = center;
			break;
		case save_practice_data:
			save_pdata = FALSE;
			break;
	}
}   


trial_data* session_config::get_trial_data(int dist, int diam, int ang) {
 	return (dtd->perfTrialGen (dist,diam,ang,collect_data == micro));
 }      
	     	
session_config::session_config(BOOL set_defaults, dtd_database* dtdd) 
	:  target_diams(), target_dists(), target_angs() {
	int *i;
	dtd_db = dtdd;    
	labels_list = dtdd->get_all_labels();
	if (set_defaults) { //need to set reasonable defaults
	//  Official defaults
		labels_list->first();	 // Just grab first available task. . .
		dtd	= dtdd->retrieve_mapping((char *)(labels_list->current()));

		i = new int; (*i) = 2 * DIST_UNIT; target_diams.append(i);
		//i = new int; (*i) = 5 * DIST_UNIT; target_diams.append(i);
		i = new int; (*i) = 10 * DIST_UNIT; target_diams.append(i);

		//i = new int; (*i) = 16 * DIST_UNIT; target_dists.append(i);
		i = new int; (*i) = 40 * DIST_UNIT; target_dists.append(i);
		i = new int; (*i) = 160 * DIST_UNIT; target_dists.append(i);

		i = new int; (*i) = 0;  target_angs.append(i);
		i = new int; (*i) = 45; target_angs.append(i);
		i = new int; (*i) = 90; target_angs.append(i);
		i = new int; (*i) = 135; target_angs.append(i);
		i = new int; (*i) = 180; target_angs.append(i);	
		i = new int; (*i) = 225; target_angs.append(i);
		i = new int; (*i) = 270; target_angs.append(i);    
		i = new int; (*i) = 315; target_angs.append(i);

		trials_per_comb = 1;
		num_blocks = 1;
		ps_len = 10;
		ps_units = practice_session::trials;
		err_tech = participant_session::redo_at_end;
		home = center;
		save_pdata = FALSE;
		collect_data = performance;
	
		//! Debugging defaults
		/*
		labels_list->first();	 // Just grab first available task. . .
		dtd	= dtdd->retrieve_mapping((char *)(labels_list->current()));

		i = new int; (*i) = 20; target_diams.append(i);
		//i = new int; (*i) = 10; target_diams.append(i);

		//i = new int; (*i) = 16; target_dists.append(i);
		i = new int; (*i) = 160; target_dists.append(i);
		//i = new int; (*i) = 40; target_dists.append(i);

		i = new int; (*i) = 270;  target_angs.append(i);
		//i = new int; (*i) = 45; target_angs.append(i);
		//i = new int; (*i) = 180; target_angs.append(i);
		//i = new int; (*i) = 135; target_angs.append(i);
		i = new int; (*i) = 90; target_angs.append(i);	
		//i = new int; (*i) = 225; target_angs.append(i);
		//i = new int; (*i) = 0; target_angs.append(i);    
		//i = new int; (*i) = 315; target_angs.append(i);

		trials_per_comb = 1;
		ps_len = 10;
		ps_units = practice_session::trials;
		err_tech = participant_session::dont_redo;
		home = center;
		save_pdata = FALSE;
		collect_data = micro;
		*/
	}
}

session_config::session_config (ifstream& infile, dtd_database* dtdd)
{
	dtd_db = dtdd;
	labels_list = dtdd->get_all_labels ();
	read_from_file (infile);
}
	
char* session_config::read_from_file(ifstream& infile) { 
	int cur_line = 0; //for providing meaningful error messages
	keyword kw;
	char *status_message = new char [80]; // for the return value 
	ostrstream sm(status_message,80); 
	char ch; // For single char gets 
	// BOOL array indexed by keyword
	BOOL kw_read_in[NUM_KEYWORDS] = 
	{FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE};
	infile.get(ch);
    while (!infile.eof()) { //read in keyword/value pairs and absorb comments
		//First, take a peek at the first char of the line, to make sure
		// it's not a comment. . .
		++cur_line;
		if (ch == '#') // This line is a comment. . .
			absorb_comment(infile);
		else   {
			infile.putback(ch); // Put back char we just peeked at
			kw = read_kw_and_vals(infile, cur_line,sm);
			if (kw == error)
				return status_message;
			else kw_read_in[(int)kw] = TRUE;
		}  
		infile.get(ch);
	} //while
	//OK, if we're here, we've successfully read in everything
	//that we're going to. Now, we need to fill in missing data. . .
	for (kw = task_type; kw <= data_to_collect; kw = (keyword)(((int)kw) + 1))
		if (!kw_read_in[(int)kw])
			fill_in_missing_kw(kw);
	sm << "SUCCESS" << ends;
	return status_message;
}

BOOL session_config::write_to_file(ofstream& outfile) {
	int success;
	outfile << "# This session configuration file was generated by method sesion_config::write_to_file.\n";
	outfile << "TASK_TYPE: " << dtd->label << "\n";
	outfile << "TARGET_DIAMETERS:"; 
	success = target_diams.first();
	while (success == TRUE) {
		outfile << " " << (*((int *)(target_diams.current()))) / DIST_UNIT; 
		success = target_diams.next();
	}
	outfile << "\n";      
	outfile << "TARGET_DISTANCES:"; 
	success = target_dists.first();
	while (success == TRUE) {
		outfile << " " << (*((int *)(target_dists.current()))) / DIST_UNIT; 
		success = target_dists.next();
	}
	outfile << "\n";     
	outfile << "TARGET_ANGLES:";
	success = target_angs.first();
	while (success == TRUE) {
		outfile << " " << (*((int *)(target_angs.current()))); 
		success = target_angs.next();
	}
	outfile << "\n";     
	outfile << "TRIALS_PER_COMBINATION: " << trials_per_comb << "\n";
	outfile << "NUMBER_OF_BLOCKS: " << num_blocks << "\n";
	outfile << "ERROR_HANDLING_TECHNIQUE: ";
	if (err_tech == participant_session::redo_at_end) 
		outfile << "redo_at_end\n";
	else
		outfile << "dont_redo\n";    
	outfile << "PRACTICE_SESSION_UNITS: ";
	if (ps_units == practice_session::trials) 
		outfile << "trials\n";
	else
		outfile << "minutes\n";
	outfile << "PRACTICE_SESSION_LENGTH: " << ps_len << "\n";
	outfile << "HOME_SQUARE_LOC: ";
	if (home == center) 
		outfile << "center\n";
	else
		outfile << "random\n";   
	outfile << "DATA_TO_COLLECT: ";
	if (collect_data == performance) 
		outfile << "performance\n";
	else
		outfile << "micro\n";
	outfile << "SAVE_PRACTICE_DATA: ";
	if (save_pdata)   
		outfile << "yes\n";
	else
		outfile << "no\n";
	return TRUE;
}
       
participant_session* session_config::generate_participant_session(int seed,
					ofstream* out, ofstream* trailsOut, const CString& partID, const CString& sessNum) { 
	trial_data *td;   
	int cur_dist, cur_diam, cur_ang, dist, diam, ang, tpc;
	int num_diams = target_diams.length();
	int num_dists = target_dists.length();
	int num_angs =	target_angs.length();
	target_dists.first(); 
	target_diams.first();  
	target_angs.first();
	cur_dist = (*(int *)(target_dists.current()));
	cur_diam = (*(int *)(target_diams.current()));
	cur_ang = (*(int *)(target_angs.current()));
	int num_t = num_diams * num_dists * num_angs * trials_per_comb;
	participant_session *ps = new participant_session (num_t, num_blocks, seed, err_tech, out, trailsOut, partID, sessNum);  
	//  Generate all the trials for a single block
	for (dist = 0; dist < num_dists; ++dist) {
		for (diam = 0; diam < num_diams; ++diam) {
			for (ang = 0; ang < num_angs; ++ang) {
				for (tpc = 0; tpc < trials_per_comb; ++tpc) {
					td = get_trial_data(cur_dist,cur_diam,cur_ang);
					ps->add_trial(td);
				}	
				target_angs.next();
				cur_ang = (*(int *)(target_angs.current()));
			}
			target_angs.first();
			cur_ang = (*(int *)(target_angs.current()));
			target_diams.next();
			cur_diam = (*(int *)(target_diams.current()));
		}
		target_diams.first();
		cur_diam = (*(int *)(target_diams.current()));
		target_dists.next();
		cur_dist = (*(int *)(target_dists.current()));
	}
	return ps;
}
		
practice_session* session_config::generate_practice_session() {
	trial_data *td;   
	int cur_dist, cur_diam, cur_ang, dist, diam, ang, tpc;
	int num_diams = target_diams.length();
	int num_dists = target_dists.length();
	int num_angs =	target_angs.length();
	target_dists.first(); 
	target_diams.first();  
	target_angs.first();
	cur_dist = (*(int *)(target_dists.current()));
	cur_diam = (*(int *)(target_diams.current()));
	cur_ang =  (*(int *)(target_angs.current()));
	int num_t = num_diams * num_dists * num_angs * trials_per_comb;
	practice_session *ps = new practice_session(num_t,ps_len,ps_units);  
	for (dist = 0; dist < num_dists; ++dist) {
		for (diam = 0; diam < num_diams; ++diam) {
			for (ang = 0; ang < num_angs; ++ang) {
				for (tpc = 0; tpc < trials_per_comb; ++tpc) {
					td = get_trial_data(cur_dist,cur_diam,cur_ang);
					ps->add_trial(td);
				}	
				target_angs.next();
				cur_ang = (*(int *)(target_angs.current()));
			}
			target_angs.first();
			cur_ang = (*(int *)(target_angs.current()));
			target_diams.next();
			cur_diam = (*(int *)(target_diams.current()));
		}
		target_diams.first();
		cur_diam = (*(int *)(target_diams.current()));
		target_dists.next();
		cur_dist = (*(int *)(target_dists.current()));
	}
	return ps;
}

experimentor_session* session_config::generate_experimentor_session() {
	trial_data *td;   
	int cur_dist, cur_diam, cur_ang, dist, diam, ang, tpc;
	int num_diams = target_diams.length();
	int num_dists = target_dists.length();
	int num_angs =	target_angs.length();
	target_dists.first(); 
	target_diams.first();  
	target_angs.first();
	cur_dist = (*(int *)(target_dists.current()));
	cur_diam = (*(int *)(target_diams.current()));
	cur_ang =  (*(int *)(target_angs.current()));
	int num_t = num_diams * num_dists * num_angs * trials_per_comb;
	experimentor_session *ps = new experimentor_session(num_t);  
	for (dist = 0; dist < num_dists; ++dist) {
		for (diam = 0; diam < num_diams; ++diam) {
			for (ang = 0; ang < num_angs; ++ang) {
				for (tpc = 0; tpc < trials_per_comb; ++tpc) {
					td = get_trial_data(cur_dist,cur_diam,cur_ang);
					ps->add_trial(td);
				}	
				target_angs.next();
				cur_ang = (*(int *)(target_angs.current()));
			}
			target_angs.first();
			cur_ang = (*(int *)(target_angs.current()));
			target_diams.next();
			cur_diam = (*(int *)(target_diams.current()));
		}
		target_diams.first();
		cur_diam = (*(int *)(target_diams.current()));
		target_dists.next();
		cur_dist = (*(int *)(target_dists.current()));
	}
	return ps;
}
  		
trial_presenter* session_config::get_tp() {
	if (collect_data == performance)
		return dtd->perf_tp->new_instance();
	else //collect_data == micro
		return dtd->micro_tp->new_instance();
}
		
session_config::~session_config() {
	delete labels_list;
}
