// PROGRAM : ses_test.cpp
// PURPOSE :  Test out the various WAPWAD classes 
// CREATED :  27 May 1996 by Chris Hundhausen

#include <string.h>
#include <iostream.h>  
#include "boolean.h"
#include "dtd_db.h" 
#include "t_data.h"
#include "s_config.h"

void handle_open_session_config(session_config *sc) {
	ofstream *dfile;
	int j;
	char cha;
	char fname[20];
	participant_session *par_ses;
  	practice_session *pra_ses;
  	experimentor_session *exp_ses;
  	trial_data *td;

	cout << "**OPERATIONS ON CONFIG FILE**\n"
		 << "Create, run, and log (P)articipant session\n"
		 << "Create, run, and log p(R)actice session\n"
		 << "Create, run, and log (E)xperimentor session\n"
		 << "(D)isplay this config file\n" 
		 << "(W)rite this config file\n"
		 << "(C)lose this config file\n\n";
	do {
		cout << "Please choose an operation (P,R,E,W,C): ";
		cin >> cha;
		switch(cha) {
	 		case 'P':
			case 'p':  
				cout << "Generating a participant session. . .\n";
				par_ses = sc->generate_participant_session();      
				cout << "Running participant session. . .\n";
				while (td = par_ses->get_next_trial())
				// Here, we would normally pass the 
				// trial on to the trial_presenter
				par_ses->log_trial(td);  
				cout << "Participant session complete!\n"
					 << "Writing data to file 'parses.dat'. . .\n";
				dfile = new ofstream("parses.dat");
				(*dfile) << (*par_ses); // Overloaded operator.
				cout << "File 'parses.dat' has been created.\n";
				delete dfile;
			break;
			case 'R':
			case 'r':
				cout << "Generating a practice session. . .\n";
				pra_ses = sc->generate_practice_session(); 
				cout << "Running participant session. . .\n";
			    while (td = pra_ses->get_next_trial()) {
			   		time_t start;
			   		time( &start);
					// Here, we would normally pass the 
					// trial on to the trial_presenter.
					// We'll assume user takes 3 sec per trial  
					while (difftime(time(NULL), start) < 3.0);
					pra_ses->log_trial(td);  
				}
				cout << "Practice session complete!\n"
				 	 << "Writing data to file 'prases.dat'. . .\n";
				dfile = new ofstream("prases.dat");
				(*dfile) << (*pra_ses); // Overloaded operator.
				delete dfile;
				cout << "File 'parses.dat' has been created.\n";
			break;
			case 'E':
			case 'e': 
				cout << "Generating an experimentor session. . .\n";
				exp_ses = sc->generate_experimentor_session(); 
				cout << "Running experimentor session with 20 trials. . .\n";  
				for (j = 0; j < 20; ++j) {
				  	td = exp_ses->get_next_trial();		    
					// Here, we would normally pass the 
					// trial on to the trial_presenter.
					// We'll assume user takes 3 sec per trial.  
					exp_ses->log_trial(td); 
				} 
				cout << "Experimentor session complete!\n"
				 	 << "Writing data to file 'expses.dat'. . .\n";
				dfile = new ofstream("expses.dat");
				(*dfile) << (*exp_ses); // Overloaded operator.
				delete dfile;
				cout << "File 'expses.dat' has been created.\n";
			break;   
			case 'W':
			case 'w':  // WRITE
				cout << "Please enter name of destination file: ";
				cin >> fname;
				dfile = new ofstream(fname);
				if (sc->write_to_file(*dfile)) 
					cout << "Configuration file " << fname << " successfully created.\n";
				else
					cout << "Configuration file " << fname << " could not be created.\n";
				delete dfile;
			break; 
			case 'D':
			case 'd': // DISPLAY
				cout << "**Parameters for current session config file**\n";
				cout << "TARGET DISTANCES  : " << sc->target_dists << "\n";
				cout << "TARGET DIAMETERS  : " << sc->target_diams << "\n";
				cout << "TARGET ANGLES     : " << sc->target_angs  << "\n";  
 				cout << "TRIALS PER COMB   : " << sc->trials_per_comb << "\n";
				cout << "ERR HANDLING TECH : ";
			if (sc->err_tech == participant_session::redo_at_end)
				cout << "redo_at_end\n";
			else 
				cout << "dont_redo\n";  
				cout << "PRACTICE SES LEN  : " << sc->ps_len << "\n";
				cout << "PRACTICE SES UNITS: ";
				if (sc->ps_units == practice_session::minutes) 
					cout << "minutes\n";
				else 
					cout << "trials\n";
				cout << "HOME SQUARE LOC   : ";
				if (sc->home == session_config::center) 
					cout << "center\n";
				else 
					cout << "random\n";
				cout << "DATA TO COLLECT   : ";
				if (sc->collect_data == session_config::performance)
					cout << "performance\n";
				else
					cout << "micro\n";
			break;
			default:
			break;  
		}  //switch
	} while (cha != 'C' && cha != 'c');
}

int main() {
  session_config *sc;
  char fname[20];
  char *result;
  char ch;  
  dtd_database *dtdd;
  dtdd = new dtd_database();
  dtd_database::data_task_def *my_dtd;
  ifstream *infile;
  my_dtd = new dtd_database::data_task_def; 
  strcpy(my_dtd->label,"discrete_pointing");
  my_dtd->perf_trial_data = new trial_data(0,0,0);
  my_dtd->micro_trial_data = NULL;
  my_dtd->perf_tp = NULL;     
  my_dtd->micro_tp = NULL;
  dtdd->add_mapping(my_dtd);  
  my_dtd = new dtd_database::data_task_def; 
  strcpy(my_dtd->label,"discrete_dragging");
  my_dtd->perf_trial_data = new trial_data(0,0,0);  
  my_dtd->micro_trial_data = NULL;
  my_dtd->perf_tp = NULL;
  my_dtd->micro_tp = NULL;
  dtdd->add_mapping(my_dtd);
  
  cout << "*TEST SESSION_CONFIG AND SESSION CLASSES*\n"
		 << "(O)pen a session configuration file\n" 
		 << "(C)reate a default session configuration file\n"
		 << "(Q)uit\n\n";
  do {
	 cout << "Please choose an operation (O,C,Q): ";
	 cin >> ch;
	 switch(ch) {
		case 'c':
		case 'C':
			sc = new session_config(TRUE,dtdd);
		 	cout << "\nSession config file with default settings created.\n";
		 	handle_open_session_config(sc);
			delete sc;
		 break;
		 case 'O':
		 case 'o':
			cout << "\nPlease enter name of config file: ";
			cin >> fname;   
			infile = new ifstream(fname);    
			if (!(*infile))
				cout << "Specified file could not be opened!\n";
			else {  
				sc = new session_config(FALSE,dtdd); 
				result = sc->read_from_file(*infile);
				delete infile;
				if (strcmp(result,"SUCCESS")==0) {
					cout << "Config file " << fname << " successfully loaded.\n";
					handle_open_session_config(sc);
					delete sc;
				}
				else {
					cout << "Config file " << infile << " could not be loaded.\n"
					     << "The following error message was returned:\n"        
					     << result << "\n";   	
				}
				delete result;
			}
		break;
		default:  //  QUIT
		break;
	} // switch
  } while (ch != 'q' && ch != 'Q');

}
