/*************************************************************************
 FILE:				SESSION.CPP    
 CREATION DATE:		19 May 1996
 LAST MODIFIED:		19 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file implements the session class, as defined in session.h
*************************************************************************/  
                
#include "stdafx.h"
#include <iostream.h>
#include <iomanip.h>

#include "winwapwad.h"

#include "session.h"

session::session(int num_trials, int blocks, int seed, ofstream* o, ofstream* trails_file, 
				 const CString& partID, const CString& sessNum)
	: pending_trials(num_trials, seed), completed_trials(),
	participantID (partID), sessionNum (sessNum)
{      
	total_trials = 0;
	trials_completed = 0;
	trials_presented = 0;
	curr_block = 0;
	num_blocks = blocks;
	mean_trial_time = 0;
	total_trial_time = 0;
	outFile = o;
	trails_out = trails_file;
	session_stopped = false;
}
		
void session::add_trial(trial_data *td) {
	td->set_session (this);
	if (block.append(td))
		++total_trials;
	else
		cout << "Error:  Couldn't enqueue a pending trial\n";	
}
	
trial_data* session::get_next_trial() {
	return ((trial_data*)(pending_trials.dequeue()));
} 

void session::write_out ()
{
	if ( ! outFile->good ())
		return;

	*outFile << *this;
}
		
/*
	The format used for output is compatible with U of Guelph's
	Generalized Fitts Law Model Builder (GFLMB) software.  In
	the case of parameters which are not recorded by this program,
	we write out default values.
*/
ofstream& operator << (ofstream& outfile, session& s) { 
	if (s.curr_block == 0)
	{
		outfile << "#  Program Version: " << CurrApp ()->Version () << "\n";
		outfile << "#  Data file:    " << s.participantID + s.sessionNum << "\n";
		outfile << "#  Subject #:    " << s.participantID << "\n";
		outfile << "#  Session #:    " << s.sessionNum << "\n";
		outfile << "#  Subject Name: (Not Recorded)\n";
		outfile << "#  Sitting #:    1\n";	// We don't record sitting number
		outfile << "#  Date/time:    " << CTime::GetCurrentTime ().Format ("%H:%M %A %B %d, %Y\n");
		outfile << "#\n";
		outfile << "#    D,     W,     H, theta,   CDG,     x,     y, E, HomeT, MoveT, Block, Seq #, Start time\n";
	}

	trial_data *lastTrial;
	s.completed_trials.first ();
	int totalTrials = s.completed_trials.length ();
	for (int i = 0; i < totalTrials; i++, s.completed_trials.next ())
	{
		lastTrial = (trial_data*)(s.completed_trials.current());
		outfile << (*lastTrial);
	}
	if (s.session_stopped)
		outfile << "# Session stopped at user request\n";
	return outfile;
}

void session::stop_session ()
{
	session_stopped = true;
}
 
	 
session::~session() {
	if (outFile != NULL)
		delete outFile;
	delete trails_out;

	clear_block ();

	block.first ();
	while (block.remove ())
	{
		block.next ();
	}
}

void session::clear_block ()
{
	trial_data *t;
	while (t = ((trial_data*)(pending_trials.dequeue()))) 
		delete t; // Dispose of this trial_data

	completed_trials.first ();
	while (completed_trials.remove())
	{
		completed_trials.next ();
	}
}

bool session::more_blocks ()
{
	return curr_block < num_blocks - 1;
}

void session::next_block ()
{
	ASSERT (curr_block < num_blocks);
	ASSERT ( ! block.empty ());
	clear_block ();

	block.first ();
	int len = block.length ();
	for (int i = 0; i < len; i++)
	{
		pending_trials.enqueue (((trial_data*) block.current ())->copy_instance ());
		block.next ();
	}

	curr_block++;
}

void session::start_first_block ()
{
	curr_block = -1;
	next_block ();
	if (trails_out != NULL)
		*trails_out << "# Program Version: " << CurrApp ()->Version () << "\n";
}
		
