/*************************************************************************
 FILE:				T_DATA.CPP   
 CREATION DATE:		5 May 1996
 LAST MODIFIED:		19 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file implements the trial_data class, as defined in t_data.h.
*************************************************************************/  

#include "stdafx.h"
#include <iostream.h>
#include <iomanip.h>

#include "session.h"

#include "t_data.h"

trial_data::trial_data(int diam, int dist, int ang, int trailSize /*= 0*/) :
		trail (trailSize)
{
			target_diameter = diam;
			target_distance = dist;
			target_angle = ang;
//			target_x = target_y = home_x = home_y = UNDEFINED; 
			trial_time = UNDEFINED;
			error = FALSE;
			nextSample = 0;
			sess = NULL;
}	

trial_data::trial_data (const trial_data& copy) :
		trail (trail)
{
	target_diameter = copy.target_diameter;
	target_distance = copy.target_distance;
	target_angle = copy.target_angle;
/*	target_x = copy.target_x;
	target_y = copy.target_y;
	home_x = copy.home_x;
	home_y = copy.home_y;
*/
	trial_time = copy.trial_time;
	error = copy.error;
	nextSample = copy.nextSample;
	sess = copy.sess;
}

trial_data* trial_data::new_instance(int diam, int dist, int ang, int trailSize) {
	return new trial_data(diam,dist,ang, trailSize);
}	

trial_data* trial_data::copy_instance ()
{
	trial_data* trial = new trial_data (*this);
	trial->sess = sess;
	return trial;
}

/*
	The format used for output is compatible with U of Guelph's
	Generalized Fitts Law Model Builder (GFLMB) software.  In
	the case of parameters which are not recorded by this program,
	we write out default values.
*/
ofstream& operator<< (ofstream& of, trial_data &td) {
	if (td.tooLarge)
		return of;		// Don't write out trials which were never run

	of << setiosflags (ios::fixed) << setprecision (1);
	of << setw (6) << td.target_distance / DIST_UNIT<< ','; 
	of << setw (6) << td.target_diameter / DIST_UNIT << ',';
	//  Repeat the diameter, becauase GFLMB format includes separate width and height
	of << setw (6) << td.target_diameter / DIST_UNIT << ',';
	of << setw (6) << td.target_angle << ',';
	of << "   1.0,";  // Control-Display Gain---not configurable for our program
	of << setw (6) << td.x_target_off << ',';
	of << setw (6) << td.y_target_off << ',';
	of << setw (2) << td.error << ',';
	of << "     0,";	// Homing time---not recorded in this version of program
	of << setw (6) << td.trial_time << ',';
	of << setw (6) << td.sess->get_curr_block () << ',';
	of << setw (6) << td.trial_seq_num << ", ";
	of << td.start_time << '\n';
	return of;
}

void trial_data::add_sample (float x, float y, int time, Sample::ButtonState button)
{
	if (is_trail ())
		trail.push_back (Sample (x, y, time, button));
}

bool trial_data::is_trail ()
{
	return trail.capacity () != 0;
}

void trial_data::write_trail (ostream& out)
{
	out << "# Amplitude: " << target_distance / DIST_UNIT << " mm, Diameter: " << target_diameter / DIST_UNIT <<
		" mm, Angle: " << target_angle << ", Sequence #: " << trial_seq_num << endl;
	out << "# Time,     X,     Y, Button" << endl;
	for (SampleVec::const_iterator forw = trail.begin ();
		 forw < trail.end ();
		 forw++)
	{
		out << *forw;
		if ( ! out.good ())
			ASSERT (0);
	}
	if (forw > trail.begin () && (forw - 1)->button == Sample::SESSION_STOPPED)
		out << "# Session stopped at user request\n";
}

ostream& operator << (ostream& out, const Sample& sample)
{

	out << setw (6) << sample.time << "," << setw (6) << sample.x << "," <<
		setw (6) << sample.y << "," << setw (7) << sample.button << endl;
	return out;
}
