/*************************************************************************
 FILE:				T_PRESEN.H    
 CREATION DATE:		27 May 1996
 LAST MODIFIED:		27 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file defines the trial_presenter class, which is responsible for 
 presenting trials to the user, interpreting input events directed at those
 trials, and ultimately recording the data of interest in trial_data 
 instances. Class trial_presenter is an abstract base class; a subclass of 
 it should be defined for each unique user task/data collection enterprise.
 Hence, implementors should take care to write subclasses of trial_presenter
 in conjunction with subclasses of trial_data, such that each trial_presenter
 subclass has a corresponding trial_data subclass capable of recording 
 the trial data of interest.
 ************************************************************************/  

#ifndef __T_PRESEN_H
#define __T_PRESEN_H
#include "t_data.h"

class trial_presenter {
	
	public:
		trial_presenter(); //Constructor
		virtual trial_presenter* new_instance() = 0; //Create a new instance of trial_presenter
		virtual void start_trial(trial_data *td) = 0;
		virtual BOOL interpret_event(unsigned event_message) = 0;
		void reset_score();
	};


#endif
