// winwadoc.cpp : implementation of the CWinwapwadDoc class
//

#include "stdafx.h"

#include "winwapwad.h"
#include "s_config.h"
#include "dtd_db.h"
#include "session.h"
#include "Point2DTrial.h"
#include "SessionID.h"
#include "CalibrateView.h"
#include "BlockCompleted.h"
#include "winwavw.h"

#include "winwadoc.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadDoc

IMPLEMENT_DYNCREATE(CWinwapwadDoc, CDocument)

BEGIN_MESSAGE_MAP(CWinwapwadDoc, CDocument)
	//{{AFX_MSG_MAP(CWinwapwadDoc)
	ON_COMMAND(ID_RUN_PARTICIPANTBLOCKSEQUENCE, OnStartParticipantSession)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadDoc construction/destruction

CWinwapwadDoc::CWinwapwadDoc()
{
	
	//!  Should initialize with my own stuff
	dtd_db = new dtd_database();
   	dtd_database::data_task_def *my_dtd;

  	my_dtd = new dtd_database::data_task_def; 
  	strcpy(my_dtd->label,"discrete_pointing");
	my_dtd->perfTrialGen = & CPoint2DTrial::NewInstance;
  	my_dtd->micro_trial_data = NULL;
  	my_dtd->perf_tp = NULL;     
  	my_dtd->micro_tp = NULL;
  	dtd_db->add_mapping(my_dtd);
	/* Someday someone should implement a dragging CView
  	my_dtd = new dtd_database::data_task_def; 
  	strcpy(my_dtd->label,"discrete_dragging");
  	my_dtd->perfTrialGen = NULL;  
  	my_dtd->micro_trial_data = NULL;
  	my_dtd->perf_tp = NULL;
  	my_dtd->micro_tp = NULL;
  	dtd_db->add_mapping(my_dtd);
	*/

	sc = NULL;
	currSess = NULL;
	currTrial = NULL;
	currView = NULL;

	// Preinitialize some graphical objects to speed up redraws
	pGreyBrush = new CBrush (RGB (192, 192, 192));
	pRedBrush = new CBrush (RGB (255, 0, 0));
}

CWinwapwadDoc::~CWinwapwadDoc()
{
	delete dtd_db;
	delete pGreyBrush;
	delete pRedBrush;
}

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadDoc serialization
//  NOTE----Never called (OnFileSave and OnFileOpen are overriden).

void CWinwapwadDoc::Serialize(CArchive& ar)
{
	CFileStatus fstatus;
	BOOL success;
	CFile *f = ar.GetFile();
	success = f->GetStatus(fstatus);
	ifstream *data_infile;
	ofstream *data_outfile;
	//fstatus.m_szFullName contains the absolute filename. . .
	//Now, we close the CFile, so that we can open it using ifstream or ofstream. . .
	f->Close();
	if (ar.IsStoring())
	{
		data_outfile = new ofstream(fstatus.m_szFullName);
		success = sc->write_to_file(*data_outfile);	
		if (success)
			MessageBox(NULL,"Session configuration successfully written!","WAPWAD",MB_OK);
		else
			MessageBox(NULL,"Session configuration could not be written!","WAPWAD",MB_OK);
		delete data_outfile;	
	}
	else
	{
		data_infile = new ifstream(fstatus.m_szFullName);
		char *msg = sc->read_from_file(*data_infile);
		MessageBox(NULL,msg,"Status of Open:",MB_OK);
		delete data_infile;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadDoc diagnostics

#ifdef _DEBUG
void CWinwapwadDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CWinwapwadDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadDoc commands

BOOL CWinwapwadDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	sc = new session_config(TRUE,dtd_db);
	currView = GetView ();
	SetSessionView ();
	return TRUE;				
}

BOOL CWinwapwadDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	ifstream in (lpszPathName);
	if ( ! in.good())
		return FALSE;

	session_config* newConfig = new session_config (FALSE, dtd_db);
	char * foo = newConfig->read_from_file (in);
	if (CString ("SUCCESS") != foo)
		return FALSE;

	DeleteContents ();
	sc = newConfig;
	SetModifiedFlag (FALSE);
	currView = GetView ();
	SetSessionView ();
	return TRUE;
}

BOOL CWinwapwadDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	ofstream out (lpszPathName);
	if ( ! out.good())
		return FALSE;

	if ( ! sc->write_to_file (out))
		return FALSE;
	
	SetModifiedFlag (FALSE);
	return TRUE;
}

void CWinwapwadDoc::DeleteContents ()
{
	delete sc;
	sc = NULL;
	delete currSess;
	currSess = NULL;
	currTrial = NULL;
	currView = NULL;
}


void CWinwapwadDoc::SetSessionView()
{
	//! Just stopgap
	//POSITION first = GetFirstViewPosition ();
	//currView = GetNextView (first);
	SetView (RUNTIME_CLASS (CWinwapwadView));
}

void CWinwapwadDoc::SetCalibrateView ()
{
	SetView (RUNTIME_CLASS (CCalibrateView));
}


/*
	The code here was taken from the commentary on the
	CDocument::AddView () member function in the
	Visual C++ Version 2.0 documentation, Vol. 3,
	pp. 272-274.

	The following sequence of calls interweaves
	both the creation of the new view and the destruction
	of the old one.  I did it this way (rather than
	grouping all the creation and all the destruction
	separately) because I was concerned about flicker.
	I'm not sure if this concern is valid or not.

	NOTE: This routine imposes a constraint on all CViews which
	might be created by it.  When the Create function is called here,
	the pointer to the CDocument for currView has not been initialized (it
	will be initialized in the call to CDocument::AddView () later).  At
	the time the window is created (i.e., immediately after Create is called),
	Win32 will send several messages to the window, which in turn MFC will
	pass on to the CView.  This will result in CView::OnSize (and perhaps 
	several other functions) getting called BEFORE the m_pDocument member
	variable has been set non-NULL.  These functions must always check that
	their m_pDocument is non-NULL before using it.

	This constraint is unavoidable;  moving the CDocument::AddView () call
	before the call to Create introduces other errors, because AddView 
	expects its CView parameter to already have a valid window.
*/
void CWinwapwadDoc::SetView (CRuntimeClass* classType)
{
	CView* oldView = currView;
	
	// Create the new CView
	currView = (CView*) classType->CreateObject ();
	ASSERT (currView != NULL && currView->IsKindOf (RUNTIME_CLASS (CView)));
	CFrameWnd* frameWindow = oldView->GetParentFrame ();
	/*
		Now I've created the CView, but it doesn't have a window associated with it.
		Call CWnd::Create to do this.
	*/
	CRect clientRect;
	frameWindow->GetClientRect ( & clientRect);
	currView->Create (NULL,		// no class (use default CWnd attributes)
					  NULL,		// no window name (we don't need one)
					  AFX_WS_DEFAULT_VIEW,	// default MFC window style
					  clientRect,	// fill the parent frame
					  frameWindow,	// the parent frame
					  AFX_IDW_PANE_FIRST, // make this window the first child, filling available space
					  NULL);		// no create context (thankfully, because I have no idea what a create context is)
	
	//CView::OnSize () will be called here by MFC (and CView::m_pDocument will be NULL)

	oldView->ShowWindow (SW_HIDE);
	currView->ShowWindow (SW_SHOW);

	// For some reason, the CDocument doesn't initialize the new view correctly unless it's
	// added BEFORE removing the old view.  I suspect the CDocument class expects to ALWAYS
	// have at least one view attached to it.
	AddView (currView);
	RemoveView (oldView);

	frameWindow->SetActiveView (currView);
	frameWindow->RecalcLayout ();

	delete oldView;
}

CView* CWinwapwadDoc::GetView ()
{
	POSITION view = GetFirstViewPosition ();
	if (view == NULL)
		return NULL;
	return GetNextView (view);
}


trial_data* CWinwapwadDoc::GetTrial ()
{
	return currTrial;
}

void CWinwapwadDoc::NextTrial ()
{
	currSess->log_trial (currTrial);
	currTrial = currSess->get_next_trial ();
	if (currTrial == NULL)
	{	
		// Following call will erase the last trial before putting the dialog box up
		UpdateAllViews (NULL);
		currSess->write_out ();
		//! Should handle file error, if any, here
		CBlockCompleted complete (currSess);
		complete.DoModal ();
		if (currSess->more_blocks ())
		{
			currSess->next_block ();
			currTrial = currSess->get_next_trial ();
		}
		else
		{
			delete currSess;
			currSess = NULL;
		}
	}
	UpdateAllViews (NULL);
}

void CWinwapwadDoc::StopSession ()
{
	currSess->log_trial (currTrial);
	currSess->write_out ();
	//! Should handle file error, if any, here
	delete currSess;
	currSess = NULL;
	currTrial = NULL;
	
	//!  TBD: Put up a dialog box telling session was cancelled at user request

	UpdateAllViews (NULL);
}

void CWinwapwadDoc::OnStartParticipantSession() 
{
	CSessionID sessDialog (sc->collect_data == session_config::micro);
	sessDialog.DoModal ();
	if (sessDialog.out == NULL)
	{
		// Couldn't open the file for some reason
		return;
	}
	CTime t = CTime::GetCurrentTime ();
	int seed = t.GetHour () + t.GetMinute () + t.GetSecond ();
	currSess = sc->generate_participant_session (seed, sessDialog.out, sessDialog.trailsOut,
							sessDialog.m_ParticipantId, sessDialog.m_SessionNum);
	currSess->start_first_block ();
	currTrial = currSess->get_next_trial ();
	UpdateAllViews (NULL);
}

void CWinwapwadDoc::CalibrationCancel ()
{
	SetSessionView ();
}

void CWinwapwadDoc::CalibrationDone (double mmToXPixels, double mmToYPixels)
{
	CurrApp ()->Calibrate (mmToXPixels, mmToYPixels);
	SetSessionView ();
}
