// winwavw.cpp : implementation of the CWinwapwadView class
//

#include "stdafx.h"
#include <math.h>

#include "winwapwad.h"
#include "s_config.h"
#include "sconfigv.h"
#include "winwadoc.h"
#include "Point2DTrial.h"
#include "TooLarge.h"

#include "winwavw.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


// Constants local to this file
const static int HOME_RADIUS = 3;		// Radius (half the width) of home rectangle, in mm
const static int MARGIN = 20;			// Distance (in mm) which home and target must be from edge ofwindow
const static int UNDEFINED_TRIAL = -100;	// Flag denoting no valid trial



/////////////////////////////////////////////////////////////////////////////
// CWinwapwadView

IMPLEMENT_DYNCREATE(CWinwapwadView, CView)

BEGIN_MESSAGE_MAP(CWinwapwadView, CView)
	//{{AFX_MSG_MAP(CWinwapwadView)
	ON_COMMAND(ID_ASPECTRATIO_SET, OnAspectratioSet)
	ON_COMMAND(ID_FILE_VIEWANDMODIFY, OnFileViewandmodify)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_SIZE()
	ON_COMMAND(ID_STOPSESSION, OnStopsession)
	ON_UPDATE_COMMAND_UI(ID_ASPECTRATIO_SET, OnUpdateAspectratioSet)
	ON_UPDATE_COMMAND_UI(ID_FILE_VIEWANDMODIFY, OnUpdateFileViewandmodify)
	ON_UPDATE_COMMAND_UI(ID_RUN_PARTICIPANTBLOCKSEQUENCE, OnUpdateRunParticipantblocksequence)
	ON_UPDATE_COMMAND_UI(ID_STOPSESSION, OnUpdateStopsession)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadView construction/destruction

CWinwapwadView::CWinwapwadView()
{
	sc_dialog = NULL;
}

CWinwapwadView::~CWinwapwadView()
{
	delete sc_dialog;
}

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadView drawing

void CWinwapwadView::OnDraw(CDC* pDC)
{
	CWinwapwadDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (homeLocation.x == UNDEFINED_TRIAL)
		return;

	if (bWaitingForHomeClick)
	{
		pDC->SelectObject (pDoc->pRedBrush);
		pDC->Rectangle (home);
		pDC->SelectObject (pDoc->pGreyBrush);
		pDC->Ellipse (target);
	//	pDC->SelectObject (hOldB);
	}
	else
	{
		/*CBrush* hOldB = */pDC->SelectObject (pDoc->pGreyBrush);
		pDC->Rectangle (home);
		pDC->SelectObject (pDoc->pRedBrush);
		pDC->Ellipse (target);
		//pDC->SelectObject (hOldB);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadView diagnostics

#ifdef _DEBUG
void CWinwapwadView::AssertValid() const
{
	CView::AssertValid();
}

void CWinwapwadView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CWinwapwadDoc* CWinwapwadView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CWinwapwadDoc)));
	return (CWinwapwadDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWinwapwadView message handlers

void CWinwapwadView::OnAspectratioSet() 
{	
	CWinwapwadDoc* doc = (CWinwapwadDoc*) GetDocument ();
	doc->SetCalibrateView ();
}

void CWinwapwadView::OnFileViewandmodify() 
// Here, we present user with the modal dialog box that elicits session_config params
{
	CWinwapwadDoc *doc = (CWinwapwadDoc*)GetDocument();
	session_config_view sc_dialog (NULL, doc->sc);
	if (sc_dialog.DoModal() == IDOK)
		doc->SetModifiedFlag (TRUE);
}


void CWinwapwadView::OnInitialUpdate() 
{
	CWinwapwadDoc *doc = (CWinwapwadDoc*)GetDocument();

	homeLocation.x = UNDEFINED_TRIAL;
	CView::OnInitialUpdate();
}

void CWinwapwadView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CWinwapwadDoc* pDoc = GetDocument();

	CPoint location (point - targetLocation);
	// Change y location from Windows coordinates (y increases going down) to 
	// conventional coordinates (y increases going up)
	location.y = - location.y;

	if (bWaitingForHomeClick)
	{
		// Are we actually in the home rectangle?
		if (home.PtInRect (point))
		{
			ulStartTime = ::GetMessageTime ();
			bWaitingForHomeClick = false;
			pCurrTrial->ClickInsideHome (PixToXMm (location.x), PixToYMm (location.y), 0, CTime::GetCurrentTime ().GetTime ());
			Invalidate ();
		}
		else
			::MessageBeep (MB_ICONEXCLAMATION);
	}
	else
	{
		int ulCurrTime = (int) (::GetMessageTime () - ulStartTime);
		if (target.PtInRect (point))
		{
			pCurrTrial->ClickInsideTarget (PixToXMm (location.x), PixToYMm (location.y), ulCurrTime);
		}
		else
		{
			pCurrTrial->ClickOutsideTarget (PixToXMm (location.x), PixToYMm (location.y), ulCurrTime);
			::MessageBeep (MB_ICONEXCLAMATION);
		}
		bWaitingForHomeClick = true;
		pDoc->NextTrial ();
	}
}

void CWinwapwadView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	OnLButtonDown (nFlags, point);
}

void CWinwapwadView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (bWaitingForHomeClick)
		return;

	int ulCurrTime = (int) (::GetMessageTime () - ulStartTime);
	CPoint location (point - targetLocation);
	// Change y location from Windows coordinates (y increases going down) to 
	// conventional coordinates (y increases going up)
	location.y = - location.y;
	pCurrTrial->MouseMove (PixToXMm (location.x), PixToYMm (location.y), ulCurrTime);
}

// Constants for use by adjust* helper functions
const int ADJUST_OK = 0;
const int ADJUST_ERR = 1;

void CWinwapwadView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CWinwapwadDoc *doc = (CWinwapwadDoc*)GetDocument();

	CObject* ob = doc->GetTrial ();
	if (ob == NULL)
	{
		homeLocation.x = UNDEFINED_TRIAL;
		Invalidate ();
		return;
	}
	ASSERT (ob->IsKindOf (RUNTIME_CLASS (CPoint2DTrial)));
	pCurrTrial = (CPoint2DTrial*) ob;

	float dist = pCurrTrial->GetTargetDistance ();
	int homeMarginYPixels = MmToYPixels (HOME_RADIUS + MARGIN);
	int homeMarginXPixels = MmToXPixels (HOME_RADIUS + MARGIN);
	targetRadius = pCurrTrial->GetTargetDiameter () / 2;
	int targetMarginYPixels = MmToYPixels (targetRadius + MARGIN);
	int targetMarginXPixels = MmToXPixels (targetRadius + MARGIN);
	int angle = pCurrTrial->GetTargetAngle ();
	CRect window;
	GetClientRect (& window);
	homeLocation.x = window.Width () / 2;
	homeLocation.y = window.Height () / 2;
	const double PI =  3.14159265359;

	targetLocation.x = homeLocation.x + MmToXPixels ((int) (cos (angle * (2 * PI / 360)) * dist));
	if (targetLocation.x - targetMarginXPixels < 0)
	{
		//           Change sign so we increase x
		if (AdjustX ( - (targetLocation.x - targetMarginXPixels),
					 homeLocation,
					 homeMarginXPixels,
					 targetLocation,
					 targetMarginXPixels,
					 window
					) != ADJUST_OK)
		{
			CancelTrial ();
			return;
		}
	}
	else if (targetLocation.x + targetMarginXPixels > window.Width ())
	{
		//           Decrease x     
		if (AdjustX (window.Width () - (targetLocation.x + targetMarginXPixels),
					 homeLocation,
					 homeMarginXPixels,
					 targetLocation,
					 targetMarginXPixels,
					 window
					) != ADJUST_OK)
		{
			CancelTrial ();
			return;
		}
	}

	// The y coordinate system has 0 at the top of the window, so we SUBTRACT the y distance
	targetLocation.y = homeLocation.y - MmToYPixels ((int) (sin (angle * (2 * PI / 360)) * dist));
	if (targetLocation.y - targetMarginYPixels < 0)
	{
		//           Change sign so we increase y
		if (AdjustY ( - (targetLocation.y - targetMarginYPixels),
					 homeLocation,
					 homeMarginYPixels,
					 targetLocation,
					 targetMarginYPixels,
					 window
					) != ADJUST_OK)
		{
			CancelTrial ();
			return;
		}
	}
	else if (targetLocation.y + targetMarginYPixels > window.Height ())
	{
		//           Decrease y    
		if (AdjustY (window.Height () - (targetLocation.y + targetMarginYPixels),
					 homeLocation,
					 homeMarginYPixels,
					 targetLocation,
					 targetMarginYPixels,
					 window
					) != ADJUST_OK)
		{
			CancelTrial ();
			return;
		}
	}

	if (doc->sc->home == session_config::random)
	{
		int leftLimit, rightLimit, direction;
		if (angle < 90 || angle > 270)
		{
			leftLimit = homeMarginXPixels;
			rightLimit = homeLocation.x;
			direction = -1;
		}
		else
		{
			leftLimit = homeLocation.x;
			rightLimit = window.Width () - homeMarginXPixels;
			direction = 1;
		}
		int offset = rand () % (rightLimit - leftLimit + 1);
		homeLocation.x += direction * offset;
		targetLocation.x += direction * offset;

		int topLimit, bottomLimit;
		if (angle > 180)
		{
			topLimit = homeMarginYPixels;
			bottomLimit = homeLocation.y;
			direction = -1;
		}
		else
		{
			topLimit = homeLocation.y;
			bottomLimit = window.Height () - homeMarginYPixels;
			direction = 1;
		}
		offset = rand () % (bottomLimit - topLimit + 1);
		homeLocation.y += direction * offset;
		targetLocation.y += direction * offset;
	}
	/*
		Every update to this CView is considered to start a new trial.  There
		may be conditions where update is called in the midst of a trial,
		but I can't think of any right now.  It seems safe to simply declare
		the beginning of a new trial.
	*/
	bWaitingForHomeClick = true;

	int homeRadiusYPixels = MmToYPixels (HOME_RADIUS);
	int homeRadiusXPixels = MmToXPixels (HOME_RADIUS);
	home = CRect (homeLocation.x - homeRadiusXPixels, homeLocation.y - homeRadiusYPixels,
				  homeLocation.x + homeRadiusXPixels, homeLocation.y + homeRadiusYPixels);

	int targetRadiusYPixels = MmToYPixels (targetRadius);
	int targetRadiusXPixels = MmToXPixels (targetRadius);
	target = CRect (targetLocation.x - targetRadiusXPixels, targetLocation.y - targetRadiusYPixels,
					targetLocation.x + targetRadiusXPixels, targetLocation.y + targetRadiusYPixels);

	SetCursorPos (homeLocation);
	Invalidate ();
}

int CWinwapwadView::AdjustX (int dist, CPoint& a, int aOffset, CPoint& b, int bOffset,
							 const CRect& window)
{
	a.x += dist;
	b.x += dist;
	if (a.x + aOffset > window.Width () || a.x - aOffset < 0 ||
		b.x + bOffset > window.Width () || b.x - bOffset < 0)
	{
		return ADJUST_ERR;
	}
	return ADJUST_OK;
}


int CWinwapwadView::AdjustY (int dist, CPoint& a, int aOffset, CPoint& b, int bOffset,
							 const CRect& window)
{
	a.y += dist;
	b.y += dist;
	if (a.y + aOffset > window.Height () || a.y - aOffset < 0 ||
		b.y + bOffset > window.Height () || b.y - bOffset < 0)
	{
		return ADJUST_ERR;
	}
	return ADJUST_OK;
}

void CWinwapwadView::CancelTrial ()
{
	homeLocation.x = UNDEFINED_TRIAL;
	pCurrTrial->TooLarge ();
	CTooLarge ().DoModal ();
	((CWinwapwadDoc*)GetDocument())->NextTrial ();
}

void CWinwapwadView::SetCursorPos (const CPoint& clientLocation)
{
	CPoint homeInScreenCoords (clientLocation);
	ClientToScreen ( & homeInScreenCoords);
	::SetCursorPos (homeInScreenCoords.x, homeInScreenCoords.y);
}


void CWinwapwadView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	if (m_pDocument != NULL)
		OnUpdate (NULL, NULL, NULL);	
}

void CWinwapwadView::OnStopsession() 
{	
	CWinwapwadDoc *pDoc = (CWinwapwadDoc*)GetDocument();
	int ulCurrTime = (int) (::GetMessageTime () - ulStartTime);
	pCurrTrial->StopSession (ulCurrTime);
	bWaitingForHomeClick = true;
	pDoc->StopSession ();
}

void CWinwapwadView::OnUpdateAspectratioSet(CCmdUI* pCmdUI) 
{
	if (ActiveSession ())
		pCmdUI->Enable (FALSE);
}

bool CWinwapwadView::ActiveSession ()
{
	CWinwapwadDoc* pDoc = (CWinwapwadDoc*) GetDocument ();
	return pDoc->GetSession () != NULL;
}

void CWinwapwadView::OnUpdateFileViewandmodify(CCmdUI* pCmdUI) 
{
	if (ActiveSession ())
		pCmdUI->Enable (FALSE);
}

void CWinwapwadView::OnUpdateRunParticipantblocksequence(CCmdUI* pCmdUI) 
{
	if (ActiveSession ())
		pCmdUI->Enable (FALSE);	
}

void CWinwapwadView::OnUpdateStopsession(CCmdUI* pCmdUI) 
{
	if ( ! ActiveSession ())
		pCmdUI->Enable (FALSE);
}
